##############################################################################
import lib_error
import logins
import pprint





##############################################################################
def new_version( error_state, pg_connection_state, rs_cluster_state ):

  if( error_state.is_error() ):
    return

  # MG2 : first check/insert cluster version
  #       constraints will fail insert if version exists

  sql = """insert into
             shared.version( version_build, version_major, version_minor )
           values
             ( %d, %d, %d );""" % ( rs_cluster_state.version_build, rs_cluster_state.version_major, rs_cluster_state.version_minor )

  pg_connection_state.issue_sql( error_state, sql=sql )

  # MG2 : quick hack - need to support no failure on existing row

  error_state.clear()

  error_state.submit_on_error( lib_error.ERR_HARD, 'arrp_shared', 'new_version', 'Failed to create new versison record.' )

  return





##############################################################################
def get_existing_cluster_postgres_id( error_state, pg_connection_state, cluster_aws_id, region_code ):

  if error_state.is_error():
    return None

  postgres_cluster_id = None

  sql = """select
             shared.cluster.cluster_postgres_id
           from
             shared.cluster,
             shared.bring_up,
             ref.region
           where
                 shared.cluster.region_id           = region.region_id
             and region.region_code                 = '%s'
             and shared.cluster.cluster_aws_id      = '%s'
             and shared.cluster.cluster_postgres_id = shared.bring_up.cluster_postgres_id
             and shared.bring_up.bring_up_success   = True
           order by
             shared.bring_up.bring_up_start_utc desc
           limit 1;""" % (region_code, cluster_aws_id)

  rows, row_count = pg_connection_state.issue_sql( error_state, sql=sql )

  if error_state.is_okay:
    postgres_cluster_id = rows[0][0]

  error_state.submit_on_error( lib_error.ERR_HARD, 'arrp_shared', 'get_existing_cluster_postgres_id', 'Failed get existing cluster postgres cluster ID.' )

  return postgres_cluster_id





##############################################################################
def new_cluster( error_state, pg_connection_state, rs_cluster_state ):

  if( error_state.is_error() ):
    return None

  cluster_postgres_id = None

  sql = '''insert into
             shared.cluster( cluster_aws_id, cluster_node_count, maintenance_track_id, node_type_id, region_id )
           select
             '%s'                                        as cluster_aws_id,
             %d                                          as cluster_node_count,
             ref.maintenance_track.maintenance_track_id  as maintenance_track_id,
             ref.node_type.node_type_id                  as node_type_id,
             ref.region.region_id                        as region_id
           from
             ref.maintenance_track,
             ref.region,
             ref.node_type
           where
                 ref.maintenance_track.maintenance_track_name = '%s'
             and ref.region.region_code                       = '%s'
             and ref.node_type.node_type_name                 = '%s'
          returning
            cluster_postgres_id;''' % ( rs_cluster_state.cluster_aws_id,
                                        rs_cluster_state.node_count,
                                        rs_cluster_state.maintenance_track,
                                        rs_cluster_state.region_code,
                                        rs_cluster_state.node_type_name )

  rows, row_count = pg_connection_state.issue_sql( error_state, sql=sql )

  postgres_cluster_id = None
  if error_state.is_okay():
    cluster_postgres_id = rows[0][0]

  error_state.submit_on_error( lib_error.ERR_HARD, 'arrp_shared', 'new_cluster', 'Failed to store new cluster information.' )

  return cluster_postgres_id





##############################################################################
def new_cluster_bring_up( error_state, pg_connection_state, rs_cluster_state, success_flag ):

  if( error_state.is_error() ):
    return

  sql = '''insert into
             shared.bring_up( bring_up_end_utc, cluster_postgres_id, bring_up_start_utc, bring_up_success )
           values
             ( '%s', %d, '%s', %s );''' % ( rs_cluster_state.most_recent_bring_up_end_utc,
                                            rs_cluster_state.cluster_postgres_id,
                                            rs_cluster_state.most_recent_bring_up_start_utc,
                                            success_flag )

  rows, row_count = pg_connection_state.issue_sql( error_state, sql=sql )

  if success_flag == True:
    new_version( error_state, pg_connection_state, rs_cluster_state )

    sql = '''insert into
               shared.cluster_info( cluster_postgres_id, version_id )
             select
               %d                         as cluster_postgres_id,
               shared.version.version_id  as version_id
             from
               shared.version
             where
                   shared.version.version_build = %d
               and shared.version.version_major = %d
               and shared.version.version_minor = %d;''' % ( rs_cluster_state.cluster_postgres_id,
                                                             rs_cluster_state.version_build,
                                                             rs_cluster_state.version_major,
                                                             rs_cluster_state.version_minor )

    rows = pg_connection_state.issue_sql( error_state, sql=sql )

  error_state.submit_on_error( lib_error.ERR_HARD, 'arrp_shared', 'new_cluster_bring_up', 'Failed to store cluster bring up information.' )

  return


