##############################################################################
import boto3
import lib_error
import json
import psycopg2
import psycopg2.extras





##############################################################################
class rs_config:

  ##############################################################################
  def __init__( self, region, rs_vpc, config_name, parameter_group_name ):

    self.__ec2c = boto3.client( 'ec2', region )
    self.__ec2r = boto3.resource( 'ec2', region )
    self.__rc   = boto3.client( 'redshift', region )

    self.__tag_name = config_name

    self.__rsn    = None
    self.__rpg    = None
    self.__rs_vpc = rs_vpc

    self.__parameter_group_name = parameter_group_name

    return

  ##############################################################################
  def create( self, error_state, queue_slots, queue_memory ):

    if error_state.is_error():
      return

    # MG2 : make and populate a Redshift subnet group

    self.__rsn = self.__tag_name
    self.__rc.create_cluster_subnet_group( ClusterSubnetGroupName = self.__rsn,
                                           Description            = 'Test Redshift Subnet Group',
                                           SubnetIds              = [ self.__rs_vpc.sn.id ] )

    # MG2 : make and populate a Redshift parameter group

    self.__rpg = self.__tag_name
    self.__create_redshift_parameter_group( error_state, queue_slots, queue_memory )

    error_state.submit_on_error( lib_error.ERR_HARD, 'rs_config', 'create', 'Failed to create Redshift config.' )

    return

  ##############################################################################
  def delete( self, error_state ):

    if error_state.is_error():
      return

    if self.__rpg != None:
      # pg_info = self.__rc.describe_cluster_parameter_groups( ParameterGroupName = self.__parameter_group_name )
      # if len(pg_info['ParameterGroups']) == 0:
        # this is an error error

      try:
        self.__rc.delete_cluster_parameter_group( ParameterGroupName = self.__parameter_group_name )
      except Exception as e:
        error_state.submit_on_error( lib_error.ERR_HARD, 'boto3', 'delete_cluster_parameter_group', 'Failed to delete Redshift parameter group, %s' % (str(e)) )

    error_state.submit_on_error( lib_error.ERR_HARD, 'rs_config', 'delete', 'Failed to delete Redshift config.' )

    return

  ##############################################################################
  def __create_redshift_parameter_group( self, error_state, queue_slots, queue_memory ):

    if error_state.is_error():
      return

    rcpg = None

    # MG2 : I may be wrong, but create_cluster_parameter_group appears to be giving a fake error 
    #       it creates the group, but throws an exception, but the exception is empty

    rcpg = self.__rc.create_cluster_parameter_group( ParameterGroupName   = self.__parameter_group_name,
                                                     ParameterGroupFamily = 'redshift-1.0',
                                                     Description          = 'Temporary parameter group' )

    # MG2 : I'm using 'wlm_json_configuration' here not because I understand how to operate this function - I do not
    #       the docs are non-existant on how you would actually use this function in practise
    #       but I managed by trial and error to figure out the (and insane) syntax for 'wlm_json_configuration' with AWS CLI
    #       and I think I can re-use that here

    parameter_json = \
    [
      {
        "query_group"           : [],
        "query_group_wild_card" : 0,
        "user_group"            : [],
        "user_group_wild_card"  : 0,
        "auto_wlm"              : False
      },
      {
        "short_query_queue" : False
      }
    ]

    # MG2 : the first queue in the list becomes the default queue and has no name
    #       which is awkward and typical of the lack of thought in Redshift design
    #       our arguments queue_slots and queue_memory describe the queues, with the first queue being the default

    for queue_id in range( 1, len(queue_slots) ):
      queue_json = {
                     "query_group"           : [ "queue_%d" % (queue_id-1) ],
                     "query_concurrency"     : queue_slots[queue_id],
                     "memory_percent_to_use" : queue_memory[queue_id]
                   }
      parameter_json.append( queue_json )

    queue_json = {
                   "query_concurrency"     : queue_slots[0],
                   "memory_percent_to_use" : queue_memory[0]
                 }
    parameter_json.append( queue_json )

    # MG2 : likewise, seems to be producing a fake error

    self.__rc.modify_cluster_parameter_group( ParameterGroupName = self.__parameter_group_name,
                                              Parameters         = \
                                              [
                                                {
                                                  'ApplyType'      : 'static',
                                                  'ParameterName'  : 'wlm_json_configuration',
                                                  'ParameterValue' : json.dumps(parameter_json),
                                                },
                                                {
                                                  'ApplyType'      : 'static',
                                                  'ParameterName'  : 'auto_analyze',
                                                  'ParameterValue' : 'false'
                                                },
                                                {
                                                  'ApplyType'      : 'static',
                                                  'ParameterName'  : 'datestyle',
                                                  'ParameterValue' : 'ISO'
                                                },
                                                {
                                                  'ApplyType'      : 'static',
                                                  'ParameterName'  : 'max_concurrency_scaling_clusters',
                                                  'ParameterValue' : '0'
                                                },
                                                {
                                                  'ApplyType'      : 'static',
                                                  'ParameterName'  : 'auto_mv',
                                                  'ParameterValue' : 'false'
                                                }
                                              ] )

    error_state.submit_on_error( lib_error.ERR_HARD, 'rs_config', '__create_redshift_parameter_group', 'Failed to create Redshift parameter group.' )

    return rcpg

